/*
 * jTunes
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.jtunes.event;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;


/**
 * Main application EventHandler and bus.  This class follows the singleton
 * pattern as there is only one in the system.  Application events are fired
 * into the handler and it will distribute all to all registered listeners
 * in the same thread.
 *
 * @author Jordan Reed
 * @version 1.0
 */
public class JTunesApplicationEventHandler {
    /** Holds the single instance of the JTunesApplicationEventHandler */
    private static JTunesApplicationEventHandler instance;

    /** Holds a set of all listeners to the application. */
    protected Set applicationEventListeners;

    /**
     * Private constructor to make the one and only one instance.
     */
    private JTunesApplicationEventHandler() {
        applicationEventListeners = new HashSet();
    }

    /**
     * Creates the event handler.  This method should be called once
     * at the launch of the application.  It is used to remove a synchronized
     * check on each call to the getInstance() method.
     *
     * @return the newly created instance of the JTunesApplicationEventHandler
     */
    public static JTunesApplicationEventHandler create() {
        instance = new JTunesApplicationEventHandler();

        return instance;
    }

    /**
     * Gets the singleton instance of the JTunesApplicationEventHandler.
     *
     * @return the singleton instance of the JTunesApplicationEventHandler
     */
    public static JTunesApplicationEventHandler getInstance() {
        return instance;
    }

    /**
     * Fires an event to all of the listeners in the current thread.
     * This method will return when all listeners have been notified
     * and take action.
     *
     * @param evt the event to fire
     */
    public void fireEvent(JTunesApplicationEvent evt) {
        Iterator applicationEventListenerIterator =
            applicationEventListeners.iterator();

        while (applicationEventListenerIterator.hasNext()) {
            JTunesApplicationEventListener applicationEventListener =
                (JTunesApplicationEventListener) applicationEventListenerIterator
                .next();
            applicationEventListener.eventPerformed(evt);
        }
    }

    /**
     * Adds an application listener to the system.
     *
     * @param applicationEventListener the listener to add
     */
    public void addJTunesApplicationEventListener(
        JTunesApplicationEventListener applicationEventListener) {
        applicationEventListeners.add(applicationEventListener);
    }

    /**
     * Gets the set of all JTunesApplicationEventListeners in the system.
     *
     * @return the set of all JTunesApplicationEventListener in the system
     */
    public Set getJTunesApplicationEventListeners() {
        return applicationEventListeners;
    }

    /**
     * Removes an application listener from the handler.
     *
     * @param applicationEventListener listener to remove
     */
    public void removeJTunesApplicationEventListener(
        JTunesApplicationEventListener applicationEventListener) {
        applicationEventListeners.remove(applicationEventListener);
    }
}
