/*
 * jTunes
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.jtunes.playlist.io;

import net.chaosserver.jtunes.event.CopySongEvent;
import net.chaosserver.jtunes.event.FinishExportEvent;
import net.chaosserver.jtunes.event.JTunesApplicationEventHandler;
import net.chaosserver.jtunes.event.StartExportEvent;
import net.chaosserver.jtunes.playlist.PlayList;
import net.chaosserver.jtunes.playlist.PlayListEntry;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import java.util.Iterator;
import java.util.List;


/**
 * Manages the loading and exporting of playlists in the M3U format.
 *
 * @author Jordan Reed
 */
public class M3uPlayListManager {
    /** Holds the character that marks a comment line in an M3U list. */
    private static final String COMMENT_START = "#";

    /**
     * Loads a playlist from an M3U file.  Currently comment lines are
     * ignored during the loading process.
     *
     * @param playListFile the file to load as an M3U PlayList
     * @return a playlist loaded from the file
     */
    public PlayList load(File playListFile) {
        PlayList playList = new PlayList();
        playList.setOriginalFileName(playListFile.getName());

        try {
            FileReader fileReader = new FileReader(playListFile);
            BufferedReader bufferedReader = new BufferedReader(fileReader);

            String nextLine = bufferedReader.readLine();

            while (nextLine != null) {
                if (
                    !nextLine.startsWith(COMMENT_START)
                        || (nextLine.trim().length() == 0)) {
                    // TODO: handle comment lines
                    PlayListEntry playListEntry =
                        new PlayListEntry(playListFile, nextLine);
                    playList.addPlayListEntry(playListEntry);
                }

                nextLine = bufferedReader.readLine();
            }

            fileReader.close();
        } catch (FileNotFoundException e) {
            throw new RuntimeException("wrapped", e);

            // TODO: Throw real exception
        } catch (IOException e) {
            throw new RuntimeException("wrapped", e);

            // TODO: Throw real exception
        }

        return playList;
    }

    /**
     * Exports the playlist to a new directory and copies all the
     * songs into that directory.  This process is done in
     * a background thread.
     *
     * @param exportDirectory directory to export the playlist and
     *        move songs into
     * @param playList the playlist to export
     */
    public void export(File exportDirectory, PlayList playList) {
        Thread exportThread =
            new Thread(new ExporterRunnable(exportDirectory, playList));
        exportThread.start();
        JTunesApplicationEventHandler.getInstance().fireEvent(
            new StartExportEvent(playList));
    }

    /**
     * Exports the playlist in the current thread.
     *
     * @param exportDirectory directory to export the playlist and
     *        move songs into
     * @param playList the playlist to export
     */
    protected void exportCurrentThread(
        File exportDirectory, PlayList playList) {
        try {
            System.out.println("Exporting to [" + exportDirectory + "]");

            File playListFile =
                new File(exportDirectory, playList.getOriginalFileName());
            System.out.println("Creating to [" + playListFile + "]");

            FileWriter fileWriter = new FileWriter(playListFile);
            BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

            List playListEntries = playList.getPlayListEntries();
            Iterator playListEntryIterator = playListEntries.iterator();

            while (playListEntryIterator.hasNext()) {
                PlayListEntry playListEntry =
                    (PlayListEntry) playListEntryIterator.next();
                String songName = playListEntry.getName();

                try {
                    System.out.println("Moving: " + songName);
                    copyFile(
                        playListEntry.getPlayListEntryFile(), exportDirectory);
                } catch (CopyException e) {
                    System.out.println("CopyException: " + e);
                    songName = "#" + songName;
                }

                System.out.println("Writing: " + songName);
                bufferedWriter.write(songName, 0, songName.length());
                bufferedWriter.newLine();
            }

            bufferedWriter.flush();
            fileWriter.close();
        } catch (IOException e) {
            throw new RuntimeException("wrapped", e);

            // TODO: Throw real exception
        } finally {
            JTunesApplicationEventHandler.getInstance().fireEvent(
                new FinishExportEvent());
        }
    }

    /**
     * Copies a file from one location to another.
     *
     * @param originalFile the original file to copy
     * @param destinationFolder the destination folder to copy the
     *        file to
     *
     * @throws CopyException indicates an error copying the file between
     *         locations
     */
    protected void copyFile(File originalFile, File destinationFolder)
        throws CopyException {
        File destinationFile =
            new File(destinationFolder, originalFile.getName());
        System.out.println(
            "Copying from [" + originalFile + "] to [" + destinationFile + "]");
        JTunesApplicationEventHandler.getInstance().fireEvent(
            new CopySongEvent());

        try {
            FileInputStream fileInputStream =
                new FileInputStream(originalFile);
            BufferedInputStream bufferedInputStream =
                new BufferedInputStream(fileInputStream);

            FileOutputStream fileOutputStream =
                new FileOutputStream(destinationFile);
            BufferedOutputStream bufferedOutputStream =
                new BufferedOutputStream(fileOutputStream);

            byte[] buffer = new byte[4096];
            int bytesRead = bufferedInputStream.read(buffer, 0, buffer.length);

            while (bytesRead >= 0) {
                bufferedOutputStream.write(buffer, 0, buffer.length);
                bytesRead = bufferedInputStream.read(buffer, 0, buffer.length);
            }

            fileInputStream.close();
            bufferedOutputStream.flush();
            fileOutputStream.close();
        } catch (FileNotFoundException e) {
            throw new CopyException("wrapped", e);
        } catch (IOException e) {
            throw new CopyException("wrapped", e);
        }
    }

    /**
     * Class used to export a playlist in a new thread.
     *
     * @author Jordan Reed
     */
    protected class ExporterRunnable implements Runnable {
        /** Holds the directory to export to. */
        protected File exportDirectory;

        /** Holds the playlist to export. */
        protected PlayList playList;

        /**
         * Constructs a new ExporterRunnable given the export information.
         *
         * @param exportDirectory directory to export the playlist to
         * @param playList playlist to export
         */
        public ExporterRunnable(File exportDirectory, PlayList playList) {
            this.exportDirectory = exportDirectory;
            this.playList = playList;
        }

        /**
         * Exports in the current thread.
         */
        public void run() {
            exportCurrentThread(this.exportDirectory, this.playList);
        }
    }
}
