/*
 * jTunes
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.jtunes.swingui;

import net.chaosserver.jtunes.event.ApplicationQuitEvent;
import net.chaosserver.jtunes.event.ApplicationStartEvent;
import net.chaosserver.jtunes.event.FileOpenEvent;
import net.chaosserver.jtunes.event.JTunesApplicationEvent;
import net.chaosserver.jtunes.event.JTunesApplicationEventHandler;
import net.chaosserver.jtunes.event.JTunesApplicationEventListener;
import net.chaosserver.jtunes.event.PlayListCloseEvent;
import net.chaosserver.jtunes.event.PlayListExportEvent;
import net.chaosserver.jtunes.event.PlayListLoadedEvent;
import net.chaosserver.jtunes.event.ShowLicenseEvent;
import net.chaosserver.jtunes.event.StartExportEvent;
import net.chaosserver.jtunes.playlist.PlayList;
import net.chaosserver.jtunes.playlist.io.PlayListManager;
import net.chaosserver.jtunes.swingui.file.FileExporter;
import net.chaosserver.jtunes.swingui.file.FileOpener;
import net.chaosserver.jtunes.swingui.license.LicensePanel;
import net.chaosserver.jtunes.swingui.menu.MainMenuBar;
import net.chaosserver.jtunes.swingui.playlist.ExportProgressDialog;
import net.chaosserver.jtunes.swingui.playlist.PlayListPanel;

import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;

import java.net.URL;

import javax.swing.JFrame;


/**
 * This is the main application class for the SwingUI.
 *
 * @author Jordan Reed
 */
public class SwingJTunes implements JTunesApplicationEventListener {
    /** Holds the application frame. */
    protected JFrame applicationFrame;

    /**
     * Holds the application logoPanel which is displayed when no
     * playlist is loaded.
     */
    protected LogoPanel logoPanel;

    /** Holds the currently loaded playlist in the frame. */
    protected PlayList playList;

    /**
     * Listens for main application events and routes to various
     * methods in the class.
     *
     * @param evt application event being fired
     */
    public void eventPerformed(JTunesApplicationEvent evt) {
        if (evt instanceof ApplicationStartEvent) {
            start();
        } else if (evt instanceof FileOpenEvent) {
            openFile();
        } else if (evt instanceof PlayListLoadedEvent) {
            displayPlayList();
        } else if (evt instanceof PlayListExportEvent) {
            exportPlayList();
        } else if (evt instanceof PlayListCloseEvent) {
            closePlayList();
        } else if (evt instanceof StartExportEvent) {
            showExportProgress((StartExportEvent) evt);
        } else if (evt instanceof ShowLicenseEvent) {
            showLicense();
        } else if (evt instanceof ApplicationQuitEvent) {
            stop();
        }
    }

    /**
     * Starts up the Swing interface.
     */
    public void start() {
        applicationFrame = new JFrame(getApplicationTitle());

        URL iconURL =
            this.getClass().getResource(
                "/net/chaosserver/jtunes/swingui/jTunesIcon.gif");
        applicationFrame.setIconImage(
            Toolkit.getDefaultToolkit().createImage(iconURL));

        logoPanel = new LogoPanel();
        applicationFrame.getContentPane().add(logoPanel);

        applicationFrame.setMenuBar(new MainMenuBar());

        applicationFrame.addWindowListener(new WindowCloser());
        applicationFrame.pack();
        applicationFrame.setVisible(true);
    }

    /**
     * Triggers a display to open a playlist.
     */
    public void openFile() {
        FileOpener fileOpener = new FileOpener(applicationFrame);
        File selectedPlaylist = fileOpener.getPlayListFile();

        if (selectedPlaylist != null) {
            PlayListManager playListManager = new PlayListManager();
            playList = playListManager.load(selectedPlaylist);
            JTunesApplicationEventHandler.getInstance().fireEvent(
                new PlayListLoadedEvent(playList));
        }
    }

    /**
     * Displays the loaded playlist.
     */
    public void displayPlayList() {
        PlayListPanel playListPanel = new PlayListPanel(playList);

        applicationFrame.getContentPane().removeAll();
        applicationFrame.getContentPane().add(playListPanel);
        applicationFrame.getContentPane().validate();
        applicationFrame.pack();
    }

    /**
     * Triggers a dialog to export the current playlist.
     */
    public void exportPlayList() {
        FileExporter fileExporter = new FileExporter(applicationFrame);
        File exportDirectory = fileExporter.getExportDirectory();

        if (exportDirectory != null) {
            System.out.println("Exporting to: " + exportDirectory);

            PlayListManager playListManager = new PlayListManager();
            playListManager.export(exportDirectory, playList);
        }
    }

    /**
     * Closes the active playlist and places the logo panel back.
     */
    public void closePlayList() {
        playList = null;
        applicationFrame.getContentPane().removeAll();
        applicationFrame.getContentPane().add(logoPanel);
        applicationFrame.getContentPane().validate();
        applicationFrame.pack();
    }

    /**
     * Displays a modal dialog with progress of the export.
     *
     * @param startExportEvent the startExportEvent to show in the progress box
     */
    public void showExportProgress(StartExportEvent startExportEvent) {
        ExportProgressDialog exportProgressDialog =
            new ExportProgressDialog(
                applicationFrame, startExportEvent.getPlayList());
        exportProgressDialog.show();
    }

    /**
     * Shows the license to the end user.
     */
    public void showLicense() {
        LicensePanel licensePanel = new LicensePanel();
        licensePanel.showLicensePanel(applicationFrame);
    }

    /**
     * Stops the application.
     */
    public void stop() {
        System.exit(0);
    }

    /**
     * Generates the application title based on the package
     * information in the manifest file.
     *
     * @return the application title.
     */
    public String getApplicationTitle() {
        Package currentPackage = getClass().getPackage();
        String specificationTitle = currentPackage.getSpecificationTitle();
        String specificationVersion = currentPackage.getSpecificationVersion();
        String implementationVersion =
            currentPackage.getImplementationVersion();

        StringBuffer sb = new StringBuffer();

        if (specificationTitle != null) {
            sb.append(specificationTitle);
        } else {
            sb.append("jTunes");
        }

        if (specificationVersion != null) {
            sb.append(" ");
            sb.append(specificationVersion);
        }

        if (implementationVersion != null) {
            sb.append(" {Build ID: ");
            sb.append(implementationVersion);
            sb.append("}");
        }

        return sb.toString();
    }

    /**
     * Basic adapter that listens for window close events
     * and stops the application.
     */
    protected class WindowCloser extends WindowAdapter {
        /**
         * Stops the application on window closing.
         *
         * @param evt the window event
         */
        public void windowClosing(WindowEvent evt) {
            stop();
        }
    }
}
