/*
 * jTunes
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.jtunes.swingui.file;

import java.awt.Component;

import java.io.File;

import java.util.prefs.Preferences;

import javax.swing.JFileChooser;


/**
 * Swing interface for retrieving a playlist to open.
 *
 * @author Jordan Reed
 */
public class FileOpener {
    /**
     * Holds the preferences constant for the last directory where a playlist
     * was opened from.
     */
    private static final String LAST_OPEN_DIRECTORY = "LAST_OPEN_DIRECTORY";

    /** Holds the parent component used to make this dialog modal. */
    protected Component parentComponent;

    /**
     * Creates a new FileOpener with the given parent component.
     *
     * @param parentComponent the component to make this modal against.
     */
    public FileOpener(Component parentComponent) {
        this.parentComponent = parentComponent;
    }

    /**
     * Displays the file chooser and return the file selected.
     *
     * @return the file selected by the chooser or null if the
     *         user did no select a file.
     */
    public File getPlayListFile() {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setMultiSelectionEnabled(false);

        File returnFile = null;
        File lastOpenDirectory = getLastOpenDirectory();

        if (lastOpenDirectory != null) {
            fileChooser.setCurrentDirectory(lastOpenDirectory);
        }

        int returnValue = fileChooser.showOpenDialog(parentComponent);

        if (returnValue == JFileChooser.APPROVE_OPTION) {
            returnFile = fileChooser.getSelectedFile();
        }

        if (returnFile != null) {
            setLastOpenDirectory(returnFile.getParentFile());
        }

        return returnFile;
    }

    /**
     * Sets the last directory that a user opened a playlist in.
     *
     * @param lastOpenDirectory the last directory the user opened
     *        a playlist in.
     */
    protected void setLastOpenDirectory(File lastOpenDirectory) {
        Preferences preferences =
            Preferences.userNodeForPackage(this.getClass());
        preferences.put(
            LAST_OPEN_DIRECTORY, lastOpenDirectory.getAbsolutePath());
    }

    /**
     * Gets the last directory that a user opened a playlist in.
     *
     * @return the last directory a user returned a playlist in or
     *         null if there is no such directory or the directory
     *         no longer exists.
     */
    protected File getLastOpenDirectory() {
        File lastOpenDirectory = null;

        Preferences preferences =
            Preferences.userNodeForPackage(this.getClass());
        String lastOpenDirectoryString =
            preferences.get(LAST_OPEN_DIRECTORY, null);

        if (lastOpenDirectoryString != null) {
            lastOpenDirectory = new File(lastOpenDirectoryString);
        }

        if ((lastOpenDirectory != null) && lastOpenDirectory.exists()) {
            return lastOpenDirectory;
        } else {
            return null;
        }
    }
}
